@@ -3,7 +3,7 @@
 // Authors:
 //   Stefan Lange (mailto:Stefan.Lange@pdfsharp.com)
 //
-// Copyright (c) 2005-2008 empira Software GmbH, Cologne (Germany)
+// Copyright (c) 2005-2009 empira Software GmbH, Cologne (Germany)
 //
 // http://www.pdfsharp.com
 // http://sourceforge.net/projects/pdfsharp
@@ -42,8 +42,9 @@ using System.Windows;
 using System.Windows.Media;
 using System.Windows.Media.Imaging;
 #endif
+using PdfSharp;
 using PdfSharp.Internal;
-using PdfSharp.Fonts.TrueType;
+using PdfSharp.Fonts.OpenType;
 using PdfSharp.Pdf;
 using PdfSharp.Pdf.IO;
 using PdfSharp.Pdf.Advanced;
@@ -80,7 +81,7 @@ namespace PdfSharp.Drawing
     }
 #endif
 
-#if WPF
+#if WPF && !SILVERLIGHT
     /// <summary>
     /// Initializes a new instance of the <see cref="XImage"/> class from a WPF image.
     /// </summary>
@@ -107,8 +108,10 @@ namespace PdfSharp.Drawing
 #if GDI
       this.gdiImage = Image.FromFile(path);
 #endif
-#if WPF
-      this.wpfImage = new BitmapImage(new Uri(path));
+#if WPF && !SILVERLIGHT
+      //BitmapSource.Create()
+      // BUG: BitmapImage locks the file
+      this.wpfImage = new BitmapImage(new Uri(path));  // AGHACK
 #endif
 
 #if false
@@ -170,7 +173,7 @@ namespace PdfSharp.Drawing
     }
 #endif
 
-#if WPF
+#if WPF && !SILVERLIGHT
     /// <summary>
     /// Conversion from BitmapSource to XImage.
     /// </summary>
@@ -246,44 +249,55 @@ namespace PdfSharp.Drawing
       }
 #endif
 #if WPF
+#if !SILVERLIGHT
       if (this.wpfImage != null)
       {
         string pixelFormat = this.wpfImage.Format.ToString();
+        string filename = GetImageFilename(this.wpfImage);
+        // WPF treats all images as images.
+        // We give JPEG images a special treatment.
+        // Test if it's a JPEG:
+        bool isJpeg = IsJpeg; // TestJpeg(filename);
+        if (isJpeg)
+        {
+          this.format = XImageFormat.Jpeg;
+          return;
+        }
+
         switch (pixelFormat)
         {
           case "Bgr32":
           case "Bgra32":
           case "Pbgra32":
-          case "{B96B3CAB-0728-11D3-9D7B-0000F81EF32E}":  // bmp
-          case "{B96B3CAF-0728-11D3-9D7B-0000F81EF32E}":  // png
             this.format = XImageFormat.Png;
             break;
 
-          case "{B96B3CAE-0728-11D3-9D7B-0000F81EF32E}":  // jpeg
-            this.format = XImageFormat.Jpeg;
-            break;
+          //case "{B96B3CAE-0728-11D3-9D7B-0000F81EF32E}":  // jpeg
+          //  this.format = XImageFormat.Jpeg;
+          //  break;
 
-          case "{B96B3CB0-0728-11D3-9D7B-0000F81EF32E}":  // gif
-          case"BlackWhite":
+          //case "{B96B3CB0-0728-11D3-9D7B-0000F81EF32E}":  // gif
+          case "BlackWhite":
           case "Indexed1":
           case "Indexed4":
           case "Indexed8":
+          case "Gray8":
             this.format = XImageFormat.Gif;
             break;
 
-          case "{B96B3CB1-0728-11D3-9D7B-0000F81EF32E}":  // tiff
-            this.format = XImageFormat.Tiff;
-            break;
+          //case "{B96B3CB1-0728-11D3-9D7B-0000F81EF32E}":  // tiff
+          //  this.format = XImageFormat.Tiff;
+          //  break;
 
-          case "{B96B3CB5-0728-11D3-9D7B-0000F81EF32E}":  // icon
-            this.format = XImageFormat.Icon;
-            break;
+          //case "{B96B3CB5-0728-11D3-9D7B-0000F81EF32E}":  // icon
+          //  this.format = XImageFormat.Icon;
+          //  break;
 
-          case "{B96B3CAC-0728-11D3-9D7B-0000F81EF32E}":  // emf
-          case "{B96B3CAD-0728-11D3-9D7B-0000F81EF32E}":  // wmf
-          case "{B96B3CB2-0728-11D3-9D7B-0000F81EF32E}":  // exif
-          case "{B96B3CB3-0728-11D3-9D7B-0000F81EF32E}":  // photoCD
-          case "{B96B3CB4-0728-11D3-9D7B-0000F81EF32E}":  // flashPIX
+          //case "{B96B3CAC-0728-11D3-9D7B-0000F81EF32E}":  // emf
+          //case "{B96B3CAD-0728-11D3-9D7B-0000F81EF32E}":  // wmf
+          //case "{B96B3CB2-0728-11D3-9D7B-0000F81EF32E}":  // exif
+          //case "{B96B3CB3-0728-11D3-9D7B-0000F81EF32E}":  // photoCD
+          //case "{B96B3CB4-0728-11D3-9D7B-0000F81EF32E}":  // flashPIX
 
           default:
             Debug.Assert(false, "Unknown pixel format: " + pixelFormat);
@@ -291,11 +305,173 @@ namespace PdfSharp.Drawing
             break;// throw new InvalidOperationException("Unsupported image format.");
         }
       }
+#else
+      // AGHACK
+#endif
 #endif
     }
 
+#if WPF
     /// <summary>
-    /// under construction
+    /// Gets the image filename.
+    /// </summary>
+    /// <param name="bitmapSource">The bitmap source.</param>
+    internal static string GetImageFilename(BitmapSource bitmapSource)
+    {
+      string filename = bitmapSource.ToString();
+      filename = UrlDecodeStringFromStringInternal(filename);
+      if (filename.StartsWith("file:///"))
+        filename = filename.Substring(8); // Remove all 3 slashes!
+      else if (filename.StartsWith("file://"))
+        filename = filename.Substring(5); // Keep 2 slashes (UNC path)
+      return filename;
+    }
+
+    private static string UrlDecodeStringFromStringInternal(string s/*, Encoding e*/)
+    {
+      int length = s.Length;
+      string result = "";
+      for (int i = 0; i < length; i++)
+      {
+        char ch = s[i];
+        if (ch == '+')
+        {
+          ch = ' ';
+        }
+        else if ((ch == '%') && (i < (length - 2)))
+        {
+          if ((s[i + 1] == 'u') && (i < (length - 5)))
+          {
+            int num3 = HexToInt(s[i + 2]);
+            int num4 = HexToInt(s[i + 3]);
+            int num5 = HexToInt(s[i + 4]);
+            int num6 = HexToInt(s[i + 5]);
+            if (((num3 < 0) || (num4 < 0)) || ((num5 < 0) || (num6 < 0)))
+            {
+              goto AddByte;
+            }
+            ch = (char)((((num3 << 12) | (num4 << 8)) | (num5 << 4)) | num6);
+            i += 5;
+            result += ch;
+            continue;
+          }
+          int num7 = HexToInt(s[i + 1]);
+          int num8 = HexToInt(s[i + 2]);
+          if ((num7 >= 0) && (num8 >= 0))
+          {
+            byte b = (byte)((num7 << 4) | num8);
+            i += 2;
+            result += (char)b;
+            continue;
+          }
+        }
+      AddByte:
+        if ((ch & 0xff80) == 0)
+        {
+          result += ch;
+        }
+        else
+        {
+          result += ch;
+        }
+      }
+      return result;
+    }
+
+    private static int HexToInt(char h)
+    {
+      if ((h >= '0') && (h <= '9'))
+      {
+        return (h - '0');
+      }
+      if ((h >= 'a') && (h <= 'f'))
+      {
+        return ((h - 'a') + 10);
+      }
+      if ((h >= 'A') && (h <= 'F'))
+      {
+        return ((h - 'A') + 10);
+      }
+      return -1;
+    }
+#endif
+
+#if WPF
+    /// <summary>
+    /// Tests if a file is a JPEG.
+    /// </summary>
+    /// <param name="filename">The filename.</param>
+    internal static bool TestJpeg(string filename)
+    {
+      byte[] imageBits = null;
+      return ReadJpegFile(filename, 16, ref imageBits);
+    }
+
+    /// <summary>
+    /// Reads the JPEG file.
+    /// </summary>
+    /// <param name="filename">The filename.</param>
+    /// <param name="maxRead">The maximum count of bytes to be read.</param>
+    /// <param name="imageBits">The bytes read from the file.</param>
+    /// <returns>False, if file could not be read or is not a JPEG file.</returns>
+    internal static bool ReadJpegFile(string filename, int maxRead, ref byte[] imageBits)
+    {
+      if (File.Exists(filename))
+      {
+        FileStream fs = null;
+        try
+        {
+          fs = new FileStream(filename, FileMode.Open, FileAccess.Read, FileShare.ReadWrite);
+        }
+        catch
+        {
+          return false;
+        }
+
+        if (fs.Length < 16)
+        {
+          fs.Close();
+          return false;
+        }
+        int len = maxRead == -1 ? (int)fs.Length : maxRead;
+        imageBits = new byte[len];
+        fs.Read(imageBits, 0, len);
+        fs.Close();
+        if (imageBits[0] == 0xff &&
+            imageBits[1] == 0xd8 &&
+            imageBits[2] == 0xff &&
+            imageBits[3] == 0xe0 &&
+            imageBits[6] == 0x4a &&
+            imageBits[7] == 0x46 &&
+            imageBits[8] == 0x49 &&
+            imageBits[9] == 0x46 &&
+            imageBits[10] == 0x0)
+        {
+          return true;
+        }
+        // TODO: Exif: find JFIF header
+        if (imageBits[0] == 0xff &&
+            imageBits[1] == 0xd8 &&
+            imageBits[2] == 0xff &&
+            imageBits[3] == 0xe1 /*&&
+            imageBits[6] == 0x4a &&
+            imageBits[7] == 0x46 &&
+            imageBits[8] == 0x49 &&
+            imageBits[9] == 0x46 &&
+            imageBits[10] == 0x0*/)
+        {
+          // Hack: store the file in PDF if extension matches ...
+          string str = filename.ToLower();
+          if (str.EndsWith(".jpg") || str.EndsWith(".jpeg"))
+            return true;
+        }
+      }
+      return false;
+    }
+#endif
+
+    /// <summary>
+    /// Under construction
     /// </summary>
     public void Dispose()
     {
@@ -319,6 +495,10 @@ namespace PdfSharp.Drawing
       }
 #endif
 #if WPF
+      if (wpfImage != null)
+      {
+        wpfImage = null;
+      }
 #endif
     }
     bool disposed;
@@ -342,7 +522,12 @@ namespace PdfSharp.Drawing
         return this.gdiImage.Width;
 #endif
 #if WPF && !GDI
+#if !SILVERLIGHT
         return this.wpfImage.PixelWidth;
+#else
+        // AGHACK
+        return 100;
+#endif
 #endif
       }
     }
@@ -365,7 +550,12 @@ namespace PdfSharp.Drawing
         return this.gdiImage.Height;
 #endif
 #if WPF && !GDI
+#if !SILVERLIGHT
         return this.wpfImage.PixelHeight;
+#else
+        // AGHACK
+        return 100;
+#endif
 #endif
       }
     }
@@ -388,8 +578,13 @@ namespace PdfSharp.Drawing
         return this.gdiImage.Width * 72 / this.gdiImage.HorizontalResolution;
 #endif
 #if WPF && !GDI
+#if !SILVERLIGHT
         Debug.Assert(Math.Abs(this.wpfImage.PixelWidth * 72 / this.wpfImage.DpiX - this.wpfImage.Width * 72.0 / 96.0) < 0.001);
         return this.wpfImage.Width * 72.0 / 96.0;
+#else
+        // AGHACK
+        return 100;
+#endif
 #endif
       }
     }
@@ -410,9 +605,14 @@ namespace PdfSharp.Drawing
 #if GDI && !WPF
         return this.gdiImage.Height * 72 / this.gdiImage.HorizontalResolution;
 #endif
-#if WPF && !GDI
+#if WPF || SILVERLIGHT && !GDI
+#if !SILVERLIGHT
         Debug.Assert(Math.Abs(this.wpfImage.PixelHeight * 72 / this.wpfImage.DpiY - this.wpfImage.Height * 72.0 / 96.0) < 0.001);
         return this.wpfImage.Height * 72.0 / 96.0;
+#else
+        // AGHACK
+        return 100;
+#endif
 #endif
       }
     }
@@ -434,7 +634,12 @@ namespace PdfSharp.Drawing
         return this.gdiImage.Width;
 #endif
 #if WPF && !GDI
+#if !SILVERLIGHT
         return this.wpfImage.PixelWidth;
+#else
+        // AGHACK
+        return 100;
+#endif
 #endif
       }
     }
@@ -456,7 +661,12 @@ namespace PdfSharp.Drawing
         return this.gdiImage.Height;
 #endif
 #if WPF && !GDI
+#if !SILVERLIGHT
         return this.wpfImage.PixelHeight;
+#else
+        // AGHACK
+        return 100;
+#endif
 #endif
       }
     }
@@ -486,7 +696,12 @@ namespace PdfSharp.Drawing
         return this.gdiImage.HorizontalResolution;
 #endif
 #if WPF && !GDI
+#if !SILVERLIGHT
         return this.wpfImage.DpiX; //.PixelWidth * 96.0 / this.wpfImage.Width;
+#else
+        // AGHACK
+        return 96;
+#endif
 #endif
       }
     }
@@ -508,7 +723,12 @@ namespace PdfSharp.Drawing
         return this.gdiImage.VerticalResolution;
 #endif
 #if WPF && !GDI
+#if !SILVERLIGHT
         return this.wpfImage.DpiY; //.PixelHeight * 96.0 / this.wpfImage.Height;
+#else
+        // AGHACK
+        return 96;
+#endif
 #endif
       }
     }
@@ -532,6 +752,92 @@ namespace PdfSharp.Drawing
     }
     XImageFormat format;
 
+#if WPF
+    /// <summary>
+    /// Gets a value indicating whether this image is JPEG.
+    /// </summary>
+    /// <value><c>true</c> if this image is JPEG; otherwise, <c>false</c>.</value>
+    public virtual bool IsJpeg
+    {
+#if !SILVERLIGHT
+      //get { if (!isJpeg.HasValue) InitializeGdiHelper(); return isJpeg.HasValue ? isJpeg.Value : false; }
+      get { if (!isJpeg.HasValue) InitializeJpegQuickTest(); return isJpeg.HasValue ? isJpeg.Value : false; }
+      //set { isJpeg = value; }
+#else
+      get { return false; } // AGHACK
+#endif
+    }
+    private bool? isJpeg;
+
+    /// <summary>
+    /// Gets a value indicating whether this image is cmyk.
+    /// </summary>
+    /// <value><c>true</c> if this image is cmyk; otherwise, <c>false</c>.</value>
+    public virtual bool IsCmyk
+    {
+#if !SILVERLIGHT
+      get { if (!isCmyk.HasValue) InitializeGdiHelper(); return isCmyk.HasValue ? isCmyk.Value : false; }
+      //set { isCmyk = value; }
+#else
+      get { return false; } // AGHACK
+#endif
+    }
+    private bool? isCmyk;
+
+#if !SILVERLIGHT
+    /// <summary>
+    /// Gets the JPEG memory stream (if IsJpeg returns true).
+    /// </summary>
+    /// <value>The memory.</value>
+    public virtual MemoryStream Memory
+    {
+      get { if (!isCmyk.HasValue) InitializeGdiHelper(); return memory; }
+      //set { memory = value; }
+    }
+    MemoryStream memory = null;
+
+    /// <summary>
+    /// Determines if an image is JPEG w/o creating an Image object.
+    /// </summary>
+    private void InitializeJpegQuickTest()
+    {
+      isJpeg = TestJpeg(GetImageFilename(wpfImage));
+    }
+
+    /// <summary>
+    /// Initializes the GDI helper.
+    /// We use GDI+ to detect if image is JPEG.
+    /// If so, we also determine if it's CMYK and we read the image bytes.
+    /// </summary>
+    private void InitializeGdiHelper()
+    {
+      if (!isCmyk.HasValue)
+      {
+        try
+        {
+          using (System.Drawing.Image image = new System.Drawing.Bitmap(GetImageFilename(wpfImage)))
+          {
+            string guid = image.RawFormat.Guid.ToString("B").ToUpper();
+            isJpeg = guid == "{B96B3CAE-0728-11D3-9D7B-0000F81EF32E}";
+            isCmyk = (image.Flags & ((int)System.Drawing.Imaging.ImageFlags.ColorSpaceCmyk | (int)System.Drawing.Imaging.ImageFlags.ColorSpaceYcck)) != 0;
+            if (isJpeg.Value)
+            {
+              memory = new MemoryStream();
+              image.Save(memory, System.Drawing.Imaging.ImageFormat.Jpeg);
+              if ((int)memory.Length == 0)
+              {
+                memory = null;
+              }
+            }
+          }
+        }
+        catch { }
+      }
+    }
+#endif    
+#endif    
+
+
 #if DEBUG_
     // TEST
     internal void CreateAllImages(string name)
