@@ -8,7 +8,7 @@
  
  / / 
  
- / /   C o p y r i g h t   ( c )   2 0 0 5 - 2 0 0 8   e m p i r a   S o f t w a r e   G m b H ,   C o l o g n e   ( G e r m a n y ) 
+ / /   C o p y r i g h t   ( c )   2 0 0 5 - 2 0 0 9   e m p i r a   S o f t w a r e   G m b H ,   C o l o g n e   ( G e r m a n y ) 
  
  / / 
  
@@ -72,7 +72,7 @@
  
  u s i n g   P d f S h a r p . D r a w i n g ; 
  
- u s i n g   P d f S h a r p . F o n t s . T r u e T y p e ; 
+ u s i n g   P d f S h a r p . F o n t s . O p e n T y p e ; 
  
  u s i n g   P d f S h a r p . P d f . F i l t e r s ; 
  
@@ -214,9 +214,29 @@
  
  
  
-             / /   C I D   f o n t s   m u s t   b e   a l w a y s   e m b e d d e d .   P D F s h a r p   e m b e d d s   a u t o m a t i c a l l y   a   s e t s e t . 
+ # i f   D E B U G 
  
-             F o n t D a t a   s u b S e t   =   t h i s . f o n t D e s c r i p t o r . d e s c r i p t o r . f o n t D a t a . C r e a t e F o n t S u b S e t ( t h i s . c m a p I n f o . G l y p h I n d i c e s ,   t r u e ) ; 
+             i f   ( t h i s . f o n t D e s c r i p t o r . d e s c r i p t o r . f o n t D a t a . l o c a   = =   n u l l ) 
+ 
+             { 
+ 
+                 G e t T y p e ( ) ; 
+ 
+             } 
+ 
+ # e n d i f 
+ 
+             / /   C I D   f o n t s   m u s t   b e   a l w a y s   e m b e d d e d .   P D F s h a r p   e m b e d d s   a u t o m a t i c a l l y   a   s u b s e t . 
+ 
+             F o n t D a t a   s u b S e t   =   n u l l ; 
+ 
+             i f   ( t h i s . f o n t D e s c r i p t o r . d e s c r i p t o r . f o n t D a t a . l o c a   = =   n u l l ) 
+ 
+                 s u b S e t   =   t h i s . f o n t D e s c r i p t o r . d e s c r i p t o r . f o n t D a t a ; 
+ 
+             e l s e 
+ 
+                 s u b S e t   =   t h i s . f o n t D e s c r i p t o r . d e s c r i p t o r . f o n t D a t a . C r e a t e F o n t S u b S e t ( t h i s . c m a p I n f o . G l y p h I n d i c e s ,   t r u e ) ; 
  
              b y t e [ ]   f o n t D a t a   =   s u b S e t . D a t a ; 
  
