@@ -8,7 +8,7 @@
  
  / / 
  
- / /   C o p y r i g h t   ( c )   2 0 0 5 - 2 0 0 8   e m p i r a   S o f t w a r e   G m b H ,   C o l o g n e   ( G e r m a n y ) 
+ / /   C o p y r i g h t   ( c )   2 0 0 5 - 2 0 0 9   e m p i r a   S o f t w a r e   G m b H ,   C o l o g n e   ( G e r m a n y ) 
  
  / / 
  
@@ -78,7 +78,7 @@
  
      / / /   < s u m m a r y > 
  
-     / / /   L e x i c a l   a n a l y s e r   f o r   P D F   c o n t e n t   f i l e s .   A d o b e   s p e c i f i e s   n o   g r a m m e r ,   b u t   i t   s e e m s   t h a t   i t 
+     / / /   L e x i c a l   a n a l y z e r   f o r   P D F   c o n t e n t   f i l e s .   A d o b e   s p e c i f i e s   n o   g r a m m a r ,   b u t   i t   s e e m s   t h a t   i t 
  
      / / /   i s   a   s i m p l e   p o s t - f i x   n o t a t i o n . 
  
@@ -254,7 +254,7 @@
  
              c h a r   c h ; 
  
-             w h i l e   ( ( c h   =   A p p e n d A n d S c a n N e x t C h a r ( ) )   ! =   C h a r s . L F   & &   c h   ! =   C h a r s . E O F )   {   } ; 
+             w h i l e   ( ( c h   =   A p p e n d A n d S c a n N e x t C h a r ( ) )   ! =   C h a r s . L F   & &   c h   ! =   C h a r s . E O F )   {   } 
  
              r e t u r n   t h i s . s y m b o l   =   C S y m b o l . C o m m e n t ; 
  
@@ -264,6 +264,28 @@
  
          / / /   < s u m m a r y > 
  
+         / / /   S c a n s   t h e   b y t e s   o f   a n   i n l i n e   i m a g e 
+ 
+         / / /   < / s u m m a r y > 
+ 
+         p u b l i c   C S y m b o l   S c a n I n l i n e I m a g e ( ) 
+ 
+         { 
+ 
+             / /   T O D O :   
+ 
+             w h i l e   ( t h i s . c u r r C h a r   ! =   ' E '   & &   t h i s . n e x t C h a r   ! =   ' I ' ) 
+ 
+                 S c a n N e x t C h a r ( ) ; 
+ 
+             r e t u r n   C S y m b o l . N o n e ; 
+ 
+         } 
+ 
+ 
+ 
+         / / /   < s u m m a r y > 
+ 
          / / /   S c a n s   a   n a m e . 
  
          / / /   < / s u m m a r y > 
@@ -286,13 +308,11 @@
  
                  i f   ( I s W h i t e S p a c e ( c h )   | |   I s D e l i m i t e r ( c h ) ) 
  
-                 { 
- 
                      r e t u r n   t h i s . s y m b o l   =   C S y m b o l . N a m e ; 
  
-                 } 
+                 
  
-                 e l s e   i f   ( c h   = =   ' # ' ) 
+                 i f   ( c h   = =   ' # ' ) 
  
                  { 
  
@@ -372,7 +392,7 @@
  
                      { 
  
-                         v a l u e   =   1 0   *   v a l u e   +   ( i n t ) c h   -   ( i n t ) ' 0 ' ; 
+                         v a l u e   =   1 0   *   v a l u e   +   c h   -   ' 0 ' ; 
  
                          i f   ( p e r i o d ) 
  
@@ -418,7 +438,7 @@
  
                  { 
  
-                     t o k e n A s R e a l   =   ( d o u b l e ) v a l u e   /   P o w e r s O f 1 0 [ d e c i m a l D i g i t s ] ; 
+                     t o k e n A s R e a l   =   v a l u e   /   P o w e r s O f 1 0 [ d e c i m a l D i g i t s ] ; 
  
                      / / t h i s . t o k e n A s L o n g   =   v a l u e   /   P o w e r s O f 1 0 [ d e c i m a l D i g i t s ] ; 
  
@@ -522,11 +542,9 @@
  
                  / /   ?   W h a t   a b o u t   \ #   e s c a p e   s e q u e n c e s 
  
-                 c h a r   c h H i ,   c h L o ; 
- 
                  S c a n N e x t C h a r ( ) ; 
  
-                 c h H i   =   S c a n N e x t C h a r ( ) ; 
+                 c h a r   c h H i   =   S c a n N e x t C h a r ( ) ; 
  
                  i f   ( c h H i   = =   ' ) ' ) 
  
@@ -540,9 +558,9 @@
  
                  } 
  
-                 c h L o   =   S c a n N e x t C h a r ( ) ; 
+                 c h a r   c h L o   =   S c a n N e x t C h a r ( ) ; 
  
-                 c h   =   ( c h a r ) ( ( i n t ) c h H i   *   2 5 6   +   ( i n t ) c h L o ) ; 
+                 c h   =   ( c h a r ) ( c h H i   *   2 5 6   +   c h L o ) ; 
  
                  w h i l e   ( t r u e ) 
  
@@ -574,9 +592,7 @@
  
                              } 
  
-                             e l s e 
- 
-                                 p a r e n L e v e l - - ; 
+                             p a r e n L e v e l - - ; 
  
                              b r e a k ; 
  
@@ -734,7 +750,7 @@
  
                      c h L o   =   S c a n N e x t C h a r ( ) ; 
  
-                     c h   =   ( c h a r ) ( ( i n t ) c h H i   *   2 5 6   +   ( i n t ) c h L o ) ; 
+                     c h   =   ( c h a r ) ( c h H i   *   2 5 6   +   c h L o ) ; 
  
                  } 
  
@@ -776,9 +792,7 @@
  
                              } 
  
-                             e l s e 
- 
-                                 p a r e n L e v e l - - ; 
+                             p a r e n L e v e l - - ; 
  
                              b r e a k ; 
  
@@ -1016,7 +1030,7 @@
  
          { 
  
-             i f   ( t h i s . C o n t L e n g t h   < =   t h i s . c h a r I n d e x ) 
+             i f   ( C o n t L e n g t h   < =   t h i s . c h a r I n d e x ) 
  
              { 
  
@@ -1046,7 +1060,7 @@
  
                          t h i s . c u r r C h a r   =   t h i s . n e x t C h a r ; 
  
-                         i f   ( t h i s . C o n t L e n g t h   < =   t h i s . c h a r I n d e x ) 
+                         i f   ( C o n t L e n g t h   < =   t h i s . c h a r I n d e x ) 
  
                              t h i s . n e x t C h a r   =   C h a r s . E O F ; 
  
@@ -1230,7 +1244,7 @@
  
          / / /   < s u m m a r y > 
  
-         / / /   I n t e r p r e t   c u r r e n t   t o k e n   a s   r e a l   o r   i n t e r g e r   l i t e r a l . 
+         / / /   I n t e r p r e t   c u r r e n t   t o k e n   a s   r e a l   o r   i n t e g e r   l i t e r a l . 
  
          / / /   < / s u m m a r y > 
  
@@ -1378,7 +1392,7 @@
  
  
  
-         b y t e [ ]   c o n t e n t ; 
+         r e a d o n l y   b y t e [ ]   c o n t e n t ; 
  
          i n t   c h a r I n d e x ; 
  
@@ -1388,7 +1402,7 @@
  
  
  
-         S t r i n g B u i l d e r   t o k e n   =   n e w   S t r i n g B u i l d e r ( ) ; 
+         r e a d o n l y   S t r i n g B u i l d e r   t o k e n   =   n e w   S t r i n g B u i l d e r ( ) ; 
  
          l o n g   t o k e n A s L o n g ; 
  
