@@ -30,6 +30,7 @@ using Hyena;
 using Hyena.Gui;
 
 using PdfSharp.Pdf;
+using PdfSharp.Pdf.IO;
 
 using PdfMod.Pdf;
 using PdfMod.Pdf.Actions;
@@ -45,11 +46,12 @@ namespace PdfMod.Gui
 
         static string [] require_doc_actions = new string[] {
             "Save", "SaveAs", "Properties", "Undo", "Redo", "ZoomFit", "OpenInViewer",
-            "SelectAll", "SelectEvens", "SelectOdds", "SelectMatching", "SelectInverse", "InsertFrom", "ExportImages"
+            "SelectAll", "SelectEvens", "SelectOdds", "SelectMatching", "SelectInverse", "InsertFrom",
+            "ViewBookmarks", "AddBookmark", "EditBookmarks"
         };
 
         static string [] require_page_actions = new string[] {
-            "Remove", "Extract", "RotateRight", "RotateLeft"
+            "Remove", "Extract", "RotateRight", "RotateLeft", "ExportImages"
         };
 
         public UndoManager UndoManager { get { return undo_manager; } }
@@ -60,52 +62,64 @@ namespace PdfMod.Gui
             undo_manager = new UndoManager ();
 
             AddImportant (
+                new ActionEntry ("FileMenu", null, Catalog.GetString ("_File"), null, null, null),
                 new ActionEntry ("Open",   Gtk.Stock.Open,   null, "<control>O", Catalog.GetString ("Open a document"), OnOpen),
-                new ActionEntry ("InsertFrom", Gtk.Stock.Add, Catalog.GetString("_Insert From..."), null, Catalog.GetString("Insert pages from another document"), OnInsertFrom),
                 new ActionEntry ("Save",   Gtk.Stock.Save,   null, "<control>S", Catalog.GetString ("Save changes to this document, overwriting the existing file"), OnSave),
                 new ActionEntry ("SaveAs", Gtk.Stock.SaveAs, null, "<control><shift>S", Catalog.GetString ("Save this document to a new file"), OnSaveAs),
-
-                new ActionEntry ("FileMenu", null, Catalog.GetString ("_File"), null, null, null),
                 new ActionEntry ("RecentMenu", null, Catalog.GetString ("Recent _Files"), null, null, null),
+                new ActionEntry ("InsertFrom", Gtk.Stock.Add, Catalog.GetString("_Insert From..."), null, Catalog.GetString("Insert pages from another document"), OnInsertFrom),
                 new ActionEntry ("Close", Gtk.Stock.Close, null, "<control>W", null, OnClose),
-                new ActionEntry ("Remove", Gtk.Stock.Remove, null, "Delete", null, OnRemove),
-                new ActionEntry ("Extract", Gtk.Stock.New, null, null, null, OnExtractPages),
-                new ActionEntry ("RotateRight", null, Catalog.GetString ("Rotate Right"), "bracketright", Catalog.GetString ("Rotate right"), OnRotateRight),
-                new ActionEntry ("RotateLeft", null, Catalog.GetString ("Rotate Left"), "bracketleft", Catalog.GetString ("Rotate left"), OnRotateLeft),
-                new ActionEntry ("ExportImages", null, Catalog.GetString ("Export Images"), null, Catalog.GetString ("Save all images in this document to a new folder"), OnExportImages),
 
                 new ActionEntry ("EditMenu", null, Catalog.GetString ("_Edit"), null, null, null),
-                new ActionEntry ("SelectAll", Stock.SelectAll, null, "<control>A", null, OnSelectAll),
-                new ActionEntry ("SelectEvens", null, Catalog.GetString ("Select Even Pages"), null, null, OnSelectEvens),
-                new ActionEntry ("SelectOdds", null, Catalog.GetString ("Select Odd Pages"), null, null, OnSelectOdds),
-                new ActionEntry ("SelectMatching", null, Catalog.GetString ("Select Matching..."), "<control>F", null, OnSelectMatching),
-                new ActionEntry ("SelectInverse", null, Catalog.GetString ("_Invert Selection"), "<shift><control>I", null, OnSelectInverse),
                 new ActionEntry ("Undo", Stock.Undo, null, "<control>z", null, OnUndo),
                 new ActionEntry ("Redo", Stock.Redo, null, "<control>y", null, OnRedo),
+                new ActionEntry ("Extract", Gtk.Stock.New, null, null, null, OnExtractPages),
+                new ActionEntry ("Remove", Gtk.Stock.Remove, null, "Delete", null, OnRemove),
+                new ActionEntry ("RotateLeft", null, Catalog.GetString ("Rotate Left"), "bracketleft", Catalog.GetString ("Rotate left"), OnRotateLeft),
+                new ActionEntry ("RotateRight", null, Catalog.GetString ("Rotate Right"), "bracketright", Catalog.GetString ("Rotate right"), OnRotateRight),
+                new ActionEntry ("ExportImages", null, null, null, null, OnExportImages),
+                new ActionEntry ("SelectAll", Stock.SelectAll, null, "<control>A", null, OnSelectAll),
+                new ActionEntry ("SelectOdds", null, Catalog.GetString ("Select Odd Pages"), null, null, OnSelectOdds),
+                new ActionEntry ("SelectEvens", null, Catalog.GetString ("Select Even Pages"), null, null, OnSelectEvens),
+                new ActionEntry ("SelectMatching", null, Catalog.GetString ("Select Matching..."), "<control>F", null, OnSelectMatching),
+                new ActionEntry ("SelectInverse", null, Catalog.GetString ("_Invert Selection"), "<shift><control>I", null, OnSelectInverse),
 
                 new ActionEntry ("ViewMenu", null, Catalog.GetString ("_View"), null, null, null),
                 new ActionEntry ("ZoomIn", Stock.ZoomIn, null, "<control>plus", null, OnZoomIn),
                 new ActionEntry ("ZoomOut", Stock.ZoomOut, null, "<control>minus", null, OnZoomOut),
+                new ActionEntry ("OpenInViewer", null, Catalog.GetString ("Open in Viewer"), "F5", Catalog.GetString ("Open in viewer"), OnOpenInViewer),
+
+                new ActionEntry ("BookmarksMenu", null, Catalog.GetString ("_Bookmarks"), null, null, null),
+                new ActionEntry ("AddBookmark", null, Catalog.GetString ("_Add Bookmark"), "<control>d", null, null),
+                new ActionEntry ("RenameBookmark", null, Catalog.GetString ("Re_name Bookmark"), "F2", null, null),
+                new ActionEntry ("ChangeBookmarkDest", null, Catalog.GetString ("_Change Bookmark Destination"), null, null, null),
+                new ActionEntry ("RemoveBookmarks", Stock.Remove, Catalog.GetString ("_Remove Bookmark"), null, null, null),
+                new ActionEntry ("EditBookmarks", null, Catalog.GetString ("_Edit Bookmarks"), "<control>B", null, OnEditBookmarks),
 
                 new ActionEntry ("HelpMenu", null, Catalog.GetString ("_Help"), null, null, null),
                 new ActionEntry ("Help", Stock.Help, Catalog.GetString ("_Contents"), "F1", null, OnHelp),
                 new ActionEntry ("About", Stock.About, null, null, null, OnAbout),
 
                 new ActionEntry ("PageContextMenu", null, "", null, null, OnPageContextMenu),
-
-                new ActionEntry ("OpenInViewer", null, Catalog.GetString ("Open in Viewer"), "F5", Catalog.GetString ("Open in viewer"), OnOpenInViewer)
+                new ActionEntry ("BookmarkContextMenu", null, "", null, null, OnBookmarkContextMenu)
             );
 
+            this["AddBookmark"].ShortLabel = Catalog.GetString ("_Add");
+            this["RemoveBookmarks"].ShortLabel = Catalog.GetString ("_Remove");
+
             AddImportant (
                 new ToggleActionEntry ("Properties", Stock.Properties, null, "<alt>Return", Catalog.GetString ("View and edit the title, keywords, and more for this document"), OnProperties, false),
                 new ToggleActionEntry ("ZoomFit", Stock.ZoomFit, null, "<control>0", null, OnZoomFit, true),
                 new ToggleActionEntry ("ViewToolbar", null, Catalog.GetString ("Toolbar"), null, null, OnViewToolbar, Client.Configuration.ShowToolbar),
+                new ToggleActionEntry ("ViewBookmarks", null, Catalog.GetString ("Bookmarks"), "F9", null, OnViewBookmarks, Client.Configuration.ShowBookmarks),
                 new ToggleActionEntry ("FullScreenView", null, Catalog.GetString ("Fullscreen"), "F11", null, OnFullScreenView, false)
             );
 
             this["RotateRight"].IconName = "object-rotate-right";
             this["RotateLeft"].IconName = "object-rotate-left";
             this["ExportImages"].IconName = "image-x-generic";
+            this["ViewBookmarks"].IconName = "user-bookmarks";
+            this["AddBookmark"].IconName = "bookmark-new";
 
             UpdateAction ("Help", true);
 
@@ -122,19 +136,10 @@ namespace PdfMod.Gui
             Register ();
 
             // Add additional menu item keybindings
-            var item = ActionManager.UIManager.GetWidget ("/MainMenu/ViewMenu/ZoomIn");
-            item.AddAccelerator ("activate", ActionManager.UIManager.AccelGroup, (uint) Gdk.Key.KP_Add, Gdk.ModifierType.ControlMask, Gtk.AccelFlags.Visible);
-            item.AddAccelerator ("activate", ActionManager.UIManager.AccelGroup, (uint) Gdk.Key.equal, Gdk.ModifierType.ControlMask, Gtk.AccelFlags.Visible);
-
-            item = ActionManager.UIManager.GetWidget ("/MainMenu/ViewMenu/ZoomOut");
-            item.AddAccelerator ("activate", ActionManager.UIManager.AccelGroup, (uint) Gdk.Key.KP_Subtract, Gdk.ModifierType.ControlMask, Gtk.AccelFlags.Visible);
-            item.AddAccelerator ("activate", ActionManager.UIManager.AccelGroup, (uint) Gdk.Key.underscore, Gdk.ModifierType.ControlMask, Gtk.AccelFlags.Visible);
-
-            item = ActionManager.UIManager.GetWidget ("/MainMenu/FileMenu/Close");
-            item.AddAccelerator ("activate", ActionManager.UIManager.AccelGroup, (uint) Gdk.Key.q, Gdk.ModifierType.ControlMask, Gtk.AccelFlags.Visible);
-
-            item = ActionManager.UIManager.GetWidget ("/MainMenu/EditMenu/Redo");
-            item.AddAccelerator ("activate", ActionManager.UIManager.AccelGroup, (uint) Gdk.Key.z, Gdk.ModifierType.ControlMask | Gdk.ModifierType.ShiftMask, Gtk.AccelFlags.Visible);
+            AddAccel ("/MainMenu/ViewMenu/ZoomIn",  Gdk.ModifierType.ControlMask, Gdk.Key.KP_Add, Gdk.Key.equal);
+            AddAccel ("/MainMenu/ViewMenu/ZoomOut", Gdk.ModifierType.ControlMask, Gdk.Key.KP_Subtract, Gdk.Key.underscore);
+            AddAccel ("/MainMenu/FileMenu/Close",   Gdk.ModifierType.ControlMask, Gdk.Key.q);
+            AddAccel ("/MainMenu/EditMenu/Redo",    Gdk.ModifierType.ControlMask | Gdk.ModifierType.ShiftMask, Gdk.Key.z);
 
             // Set up recent documents menu
             MenuItem recent_item = ActionManager.UIManager.GetWidget ("/MainMenu/FileMenu/RecentMenu") as MenuItem;
@@ -149,6 +154,14 @@ namespace PdfMod.Gui
             recent_item.Submenu = recent_chooser_item;
         }
 
+        private void AddAccel (string path, Gdk.ModifierType modifier, params Gdk.Key [] keys)
+        {
+            var item = ActionManager.UIManager.GetWidget (path);
+            foreach (var key in keys) {
+                item.AddAccelerator ("activate", ActionManager.UIManager.AccelGroup, (uint)key, modifier, Gtk.AccelFlags.Visible);
+            }
+        }
+
         void OnChanged (object o, EventArgs args)
         {
             Update ();
@@ -164,6 +177,17 @@ namespace PdfMod.Gui
             foreach (string action in require_page_actions)
                 UpdateAction (action, true, have_page);
 
+            int exportable_imgs = 0;
+            if (have_page) {
+                exportable_imgs = new ExportImagesAction (app.Document, app.IconView.SelectedPages).ExportableImageCount;
+            }
+            UpdateAction ("ExportImages", true, exportable_imgs > 0);
+            this["ExportImages"].Label = String.Format (Catalog.GetPluralString (
+                "Export Image", "Export {0} Images", exportable_imgs), exportable_imgs);
+
+            this["ExportImages"].Tooltip = String.Format (Catalog.GetPluralString (
+                "Save image from the selected pages to a new folder", "Save {0} images from the selected pages to a new folder", exportable_imgs), exportable_imgs);
+
             UpdateAction ("Undo", true, have_doc && undo_manager.CanUndo);
             UpdateAction ("Redo", true, have_doc && undo_manager.CanRedo);
 
@@ -177,7 +201,7 @@ namespace PdfMod.Gui
                 ? Catalog.GetString ("_Redo")
                 : String.Format (Catalog.GetString ("Redo {0}"), redo.Description);
 
-            UpdateActions (true, have_doc && app.Document.HasUnsavedChanged, "Save", "SaveAs");
+            UpdateAction ("Save", true, have_doc && app.Document.HasUnsavedChanges);
             UpdateAction ("ZoomIn", true, have_doc && app.IconView.CanZoomIn);
             UpdateAction ("ZoomOut", true, have_doc && app.IconView.CanZoomOut);
 
@@ -198,6 +222,8 @@ namespace PdfMod.Gui
 
 #region Action Handlers
 
+        // File menu actions
+
         void OnOpen (object o, EventArgs args)
         {
             var chooser = app.CreateChooser (Catalog.GetString ("Select PDF"), FileChooserAction.Open);
@@ -223,22 +249,68 @@ namespace PdfMod.Gui
             }
         }
 
-        void OnOpenInViewer (object o, EventArgs args)
+        void OnSave (object o, EventArgs args)
         {
-            System.Diagnostics.Process.Start (app.Document.CurrentStateUri);
+            try {
+                app.Document.Save (app.Document.SuggestedSavePath);
+                undo_manager.Clear ();
+            } catch (Exception e) {
+                Log.Error (Catalog.GetString ("Unable to Save Document"), e.Message, true);
+            }
         }
 
-        void OnFullScreenView (object o, EventArgs args)
+        void OnSaveAs (object o, EventArgs args)
         {
-            bool fullscreen = (this["FullScreenView"] as ToggleAction).Active;
+            var chooser = app.CreateChooser (Catalog.GetString ("Save as..."), FileChooserAction.Save);
+            chooser.SelectMultiple = false;
+            chooser.DoOverwriteConfirmation = true;
+            chooser.CurrentName = System.IO.Path.GetFileName (app.Document.SuggestedSavePath);
+            chooser.AddButton (Stock.SaveAs, ResponseType.Ok);
+            chooser.SetCurrentFolder (System.IO.Path.GetDirectoryName (app.Document.SuggestedSavePath));
 
-            if (fullscreen) {
-                app.Window.Fullscreen ();
-            } else {
-                app.Window.Unfullscreen ();
+            var response = chooser.Run ();
+            string filename = chooser.Filename;
+            chooser.Destroy ();
+
+            if (response == (int)ResponseType.Ok) {
+                Log.DebugFormat ("Saving {0} to {1}", app.Document.Uri, filename);
+                app.Document.Save (filename);
+                undo_manager.Clear ();
             }
         }
 
+        void OnExportImages (object o, EventArgs args)
+        {
+            var pages = app.IconView.SelectedPages.ToList ();
+            var action = new ExportImagesAction (app.Document, pages);
+            if (action.ExportableImageCount == 0) {
+                Log.Information ("Found zero exportable images in the selected pages");
+                return;
+            }
+
+            var export_path_base = Path.Combine (
+                Path.GetDirectoryName (app.Document.SuggestedSavePath),
+                Hyena.StringUtil.EscapeFilename (String.Format ("{0} [{1}]",
+                    Path.GetFileNameWithoutExtension (app.Document.SuggestedSavePath),
+                    GLib.Markup.EscapeText (Document.GetPageSummary (pages, 10))))
+            );
+
+            var export_path = export_path_base;
+            int i = 1;
+            while (Directory.Exists (export_path) && i < 100) {
+                export_path = String.Format ("{0} ({1})", export_path_base, i++);
+            }
+
+            try {
+                Directory.CreateDirectory (export_path);
+            } catch (Exception e) {
+                Hyena.Log.Exception (e);
+            }
+
+            action.Do (export_path);
+            System.Diagnostics.Process.Start (export_path);
+        }
+
         void OnInsertFrom (object o, EventArgs args)
         {
             var chooser = app.CreateChooser (Catalog.GetString ("Select PDF"), FileChooserAction.Open);
@@ -265,32 +337,6 @@ namespace PdfMod.Gui
             }
         }
 
-        void OnSave (object o, EventArgs args)
-        {
-            app.Document.Save (app.Document.SuggestedSavePath);
-            undo_manager.Clear ();
-        }
-
-        void OnSaveAs (object o, EventArgs args)
-        {
-            var chooser = app.CreateChooser (Catalog.GetString ("Save as..."), FileChooserAction.Save);
-            chooser.SelectMultiple = false;
-            chooser.DoOverwriteConfirmation = true;
-            chooser.CurrentName = System.IO.Path.GetFileName (app.Document.SuggestedSavePath);
-            chooser.AddButton (Stock.SaveAs, ResponseType.Ok);
-            chooser.SetCurrentFolder (System.IO.Path.GetDirectoryName (app.Document.SuggestedSavePath));
-
-            var response = chooser.Run ();
-            string filename = chooser.Filename;
-            chooser.Destroy ();
-
-            if (response == (int)ResponseType.Ok) {
-                Log.DebugFormat ("Saving {0} to {1}", app.Document.Uri, filename);
-                app.Document.Save (filename);
-                undo_manager.Clear ();
-            }
-        }
-
         void OnProperties (object o, EventArgs args)
         {
             app.EditorBox.Visible = (this["Properties"] as ToggleAction).Active;
@@ -299,64 +345,12 @@ namespace PdfMod.Gui
             }
         }
 
-        void OnRemove (object o, EventArgs args)
+        void OnClose (object o, EventArgs args)
         {
-            var action = new RemoveAction (app.Document, app.IconView.SelectedPages);
-            action.Do ();
-            // Undo isn't working yet
-            //undo_manager.AddUndoAction (action);
+            app.Quit ();
         }
 
-        void OnExtractPages (object o, EventArgs args)
-        {
-            var doc = new PdfDocument ();
-            var pages = new List<Page> (app.IconView.SelectedPages);
-            foreach (var page in pages) {
-                doc.AddPage (page.Pdf);
-            }
-
-            var path = Client.GetTmpFilename ();
-            doc.Save (path);
-            doc.Dispose ();
-
-            app.LoadPath (path, Path.Combine (
-                Path.GetDirectoryName (app.Document.SuggestedSavePath),
-                String.Format ("[{0}] {1}",
-                    GLib.Markup.EscapeText (Document.GetPageSummary (pages, 10)),
-                    Path.GetFileName (app.Document.SuggestedSavePath))
-            ));
-        }
-
-        void OnExportImages (object o, EventArgs args)
-        {
-            var action = new ExportImagesAction (app.Document, app.Document.Pages);
-            if (action.ExportableImageCount == 0) {
-                Log.Information ("Found zero exportable images in the selected pages");
-                return;
-            }
-
-            var export_path_base = Path.Combine (
-                Path.GetDirectoryName (app.Document.SuggestedSavePath),
-                // Translators: This is used for creating a folder name, be careful!
-                //String.Format (Catalog.GetString ("{0} - Images for {1}"), app.Document.Title ?? app.Document.Filename, GetPageSummary (pages, 10))
-                app.Document.Title ?? System.IO.Path.GetFileNameWithoutExtension (app.Document.Filename)
-            );
-
-            var export_path = export_path_base;
-            int i = 1;
-            while (Directory.Exists (export_path) && i < 100) {
-                export_path = String.Format ("{0} ({1})", export_path_base, i++);
-            }
-
-            try {
-                Directory.CreateDirectory (export_path);
-            } catch (Exception e) {
-                Hyena.Log.Exception (e);
-            }
-
-            action.Do (export_path);
-            System.Diagnostics.Process.Start (export_path);
-        }
+        // Edit menu actions
 
         void OnUndo (object o, EventArgs args)
         {
@@ -368,18 +362,121 @@ namespace PdfMod.Gui
             undo_manager.Redo ();
         }
 
-        [DllImport ("glib-2.0.dll")]
-        static extern IntPtr g_get_language_names ();
-
-        string CombinePaths (params string [] parts)
+        void OnExtractPages (object o, EventArgs args)
         {
-            string path = parts[0];
-            for (int i = 1; i < parts.Length; i++) {
-                path = System.IO.Path.Combine (path, parts[i]);
+            var to_doc = new PdfDocument ();
+            var from_doc = PdfSharp.Pdf.IO.PdfReader.Open (new Uri (app.Document.CurrentStateUri).LocalPath, PdfDocumentOpenMode.Import, null);
+            var pages = app.IconView.SelectedPages.ToList ();
+
+            foreach (var index in pages.Select (p => p.Index)) {
+                to_doc.AddPage (from_doc.Pages[index]);
             }
-            return path;
+
+            var path = Client.GetTmpFilename ();
+            to_doc.Save (path);
+            to_doc.Dispose ();
+
+            app.LoadPath (path, Path.Combine (
+                Path.GetDirectoryName (app.Document.SuggestedSavePath),
+                Hyena.StringUtil.EscapeFilename (String.Format ("{0} [{1}].pdf",
+                    Path.GetFileNameWithoutExtension (app.Document.SuggestedSavePath),
+                    GLib.Markup.EscapeText (Document.GetPageSummary (pages, 10))))
+            ));
         }
 
+        void OnRemove (object o, EventArgs args)
+        {
+            var action = new RemoveAction (app.Document, app.IconView.SelectedPages);
+            action.Do ();
+            // Undo isn't working yet
+            //undo_manager.AddUndoAction (action);
+        }
+
+        void OnRotateLeft (object o, EventArgs args)
+        {
+            Rotate (-90);
+        }
+
+        void OnRotateRight (object o, EventArgs args)
+        {
+            Rotate (90);
+        }
+
+        void OnSelectAll (object o, EventArgs args)
+        {
+            app.IconView.SetPageSelectionMode (PageSelectionMode.All);
+        }
+
+        void OnSelectOdds (object o, EventArgs args)
+        {
+            app.IconView.SetPageSelectionMode (PageSelectionMode.Odds);
+        }
+
+        void OnSelectEvens (object o, EventArgs args)
+        {
+            app.IconView.SetPageSelectionMode (PageSelectionMode.Evens);
+        }
+
+        void OnSelectMatching (object o, EventArgs args)
+        {
+            app.ToggleMatchQuery ();
+        }
+
+        void OnSelectInverse (object o, EventArgs args)
+        {
+            app.IconView.SetPageSelectionMode (PageSelectionMode.Inverse);
+        }
+
+        // View menu actions
+
+        void OnZoomIn (object o, EventArgs args)
+        {
+            app.IconView.Zoom (10);
+        }
+
+        void OnZoomOut (object o, EventArgs args)
+        {
+            app.IconView.Zoom (-10);
+        }
+
+        void OnZoomFit (object o, EventArgs args)
+        {
+            app.IconView.ZoomFit ();
+        }
+
+        void OnOpenInViewer (object o, EventArgs args)
+        {
+            System.Diagnostics.Process.Start (app.Document.CurrentStateUri);
+        }
+
+        void OnFullScreenView (object o, EventArgs args)
+        {
+            bool fullscreen = (this["FullScreenView"] as ToggleAction).Active;
+
+            if (fullscreen) {
+                app.Window.Fullscreen ();
+            } else {
+                app.Window.Unfullscreen ();
+            }
+        }
+
+        void OnViewToolbar (object o, EventArgs args)
+        {
+            bool show = (this["ViewToolbar"] as ToggleAction).Active;
+            Client.Configuration.ShowToolbar = app.HeaderToolbar.Visible = show;
+        }
+
+        void OnViewBookmarks (object o, EventArgs args)
+        {
+            bool show = (this["ViewBookmarks"] as ToggleAction).Active;
+            Client.Configuration.ShowBookmarks = app.BookmarkView.Visible = show;
+            if (app.BookmarkView.Visible) {
+                app.BookmarkView.GrabFocus ();
+            }
+        }
+
+        // Help menu actions
+
         void OnHelp (object o, EventArgs args)
         {
             bool shown = false;
@@ -448,7 +545,10 @@ namespace PdfMod.Gui
                     "\tŁukasz Jernaś",
                     "\tRomain Tartière",
                     "\tRobert Dyer",
-                    "\tAndreu Correa Casablanca"
+                    "\tAndreu Correa Casablanca",
+                    "\tChow Loong Jin",
+                    "\tRobson Roberto Souza Peixoto",
+                    "\tThomas Klausner"
                 },
                 Documenters = new string [] { "Gabriel Burt" },
                 Artists = new string [] { "Kalle Persson" },
@@ -456,7 +556,7 @@ namespace PdfMod.Gui
                     // Translators: {0} and {1} are the years the copyright assertion covers; put into
                     // variables so you don't have to re-translate this every year
                     Catalog.GetString ("Copyright {0} Novell Inc.\nCopyright {1} Other PDF Mod Contributors"),
-                    "2009-2010", "2009"
+                    "2009-2011", "2009-2011"
                 ),
                 TranslatorCredits = Catalog.GetString ("translator-credits")
             };
@@ -488,61 +588,23 @@ namespace PdfMod.Gui
             ShowContextMenu ("/PageContextMenu");
         }
 
-        void OnSelectAll (object o, EventArgs args)
+        // Bookmark actions
+
+        void OnEditBookmarks (object o, EventArgs args)
         {
-            app.IconView.SetPageSelectionMode (PageSelectionMode.All);
+            if (!app.BookmarkView.Visible) {
+                (this["ViewBookmarks"] as ToggleAction).Active = true;
+            }
         }
 
-        void OnSelectEvens (object o, EventArgs args)
+        void OnBookmarkContextMenu (object o, EventArgs args)
         {
-            app.IconView.SetPageSelectionMode (PageSelectionMode.Evens);
+            ShowContextMenu ("/BookmarkContextMenu");
         }
 
-        void OnSelectOdds (object o, EventArgs args)
-        {
-            app.IconView.SetPageSelectionMode (PageSelectionMode.Odds);
-        }
+#endregion
 
-        void OnSelectMatching (object o, EventArgs args)
-        {
-            app.ToggleMatchQuery ();
-        }
-
-        void OnSelectInverse (object o, EventArgs args)
-        {
-            app.IconView.SetPageSelectionMode (PageSelectionMode.Inverse);
-        }
-
-        void OnZoomIn (object o, EventArgs args)
-        {
-            app.IconView.Zoom (10);
-        }
-
-        void OnZoomOut (object o, EventArgs args)
-        {
-            app.IconView.Zoom (-10);
-        }
-
-        void OnZoomFit (object o, EventArgs args)
-        {
-            app.IconView.ZoomFit ();
-        }
-
-        void OnViewToolbar (object o, EventArgs args)
-        {
-            bool show = (this["ViewToolbar"] as ToggleAction).Active;
-            Client.Configuration.ShowToolbar = app.HeaderToolbar.Visible = show;
-        }
-
-        void OnRotateRight (object o, EventArgs args)
-        {
-            Rotate (90);
-        }
-
-        void OnRotateLeft (object o, EventArgs args)
-        {
-            Rotate (-90);
-        }
+#region Utility methods
 
         void Rotate (int degrees)
         {
@@ -553,12 +615,29 @@ namespace PdfMod.Gui
             }
         }
 
-        void OnClose (object o, EventArgs args)
+        string CombinePaths (params string [] parts)
         {
-            app.Quit ();
+            string path = parts[0];
+            for (int i = 1; i < parts.Length; i++) {
+                path = System.IO.Path.Combine (path, parts[i]);
+            }
+            return path;
         }
 
+        [DllImport ("glib-2.0.dll")]
+        static extern IntPtr g_get_language_names ();
+
 #endregion
 
     }
+
+    public static class ActionExtensions
+    {
+        public static Hyena.Widgets.ImageButton CreateImageButton (this Gtk.Action action)
+        {
+            var button = new Hyena.Widgets.ImageButton (action.ShortLabel, action.IconName);
+            action.ConnectProxy (button);
+            return button;
+        }
+    }
 }
