@@ -110,7 +110,7 @@ namespace PdfMod.Gui
         void HandleDocumentLoaded (object o, EventArgs e)
         {
             doc = app.Document;
-            var pdf = app.Document.Pdf;
+            /*var pdf = app.Document.Pdf;
             var info = pdf.Info;
             
             Console.WriteLine ("Author           = {0}", info.Author);
@@ -140,7 +140,7 @@ namespace PdfMod.Gui
             Console.WriteLine ("FitWindow        = {0}", prefs.FitWindow);
             Console.WriteLine ("HideMenubar      = {0}", prefs.HideMenubar);
             Console.WriteLine ("HideToolbar      = {0}", prefs.HideToolbar);
-            Console.WriteLine ("HideWindowUI     = {0}", prefs.HideWindowUI);
+            Console.WriteLine ("HideWindowUI     = {0}", prefs.HideWindowUI);*/
 
             title.SetDefault (doc.Title);
             author.SetDefault (doc.Author);
